@@ -1,7 +1,8 @@
+from __future__ import annotations
+
 from collections import deque
 from dataclasses import dataclass
 from types import TracebackType
-from typing import Deque, Optional, Tuple, Type
 from warnings import warn
 
 from ..lowlevel import cancel_shielded_checkpoint, checkpoint, checkpoint_if_cancelled
@@ -34,7 +35,7 @@ class CapacityLimiterStatistics:
 
     borrowed_tokens: int
     total_tokens: float
-    borrowers: Tuple[object, ...]
+    borrowers: tuple[object, ...]
     tasks_waiting: int
 
 
@@ -48,7 +49,7 @@ class LockStatistics:
     """
 
     locked: bool
-    owner: Optional[TaskInfo]
+    owner: TaskInfo | None
     tasks_waiting: int
 
 
@@ -69,11 +70,12 @@ class SemaphoreStatistics:
     :ivar int tasks_waiting: number of tasks waiting on :meth:`~.Semaphore.acquire`
 
     """
+
     tasks_waiting: int
 
 
 class Event:
-    def __new__(cls):
+    def __new__(cls) -> Event:
         return get_asynclib().Event()
 
     def set(self) -> DeprecatedAwaitable:
@@ -84,7 +86,7 @@ class Event:
         """Return ``True`` if the flag is set, ``False`` if not."""
         raise NotImplementedError
 
-    async def wait(self) -> bool:
+    async def wait(self) -> None:
         """
         Wait until the flag has been set.
 
@@ -99,17 +101,20 @@ class Event:
 
 
 class Lock:
-    _owner_task: Optional[TaskInfo] = None
+    _owner_task: TaskInfo | None = None
 
-    def __init__(self):
-        self._waiters: Deque[Tuple[TaskInfo, Event]] = deque()
+    def __init__(self) -> None:
+        self._waiters: deque[tuple[TaskInfo, Event]] = deque()
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> None:
         await self.acquire()
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     async def acquire(self) -> None:
@@ -127,23 +132,29 @@ class Lock:
             except BaseException:
                 if not event.is_set():
                     self._waiters.remove(token)
+                elif self._owner_task == task:
+                    self.release()
 
                 raise
 
             assert self._owner_task == task
         else:
-            await cancel_shielded_checkpoint()
+            try:
+                await cancel_shielded_checkpoint()
+            except BaseException:
+                self.release()
+                raise
 
     def acquire_nowait(self) -> None:
         """
         Acquire the lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         task = get_current_task()
         if self._owner_task == task:
-            raise RuntimeError('Attempted to acquire an already held Lock')
+            raise RuntimeError("Attempted to acquire an already held Lock")
 
         if self._owner_task is not None:
             raise WouldBlock
@@ -153,7 +164,7 @@ class Lock:
     def release(self) -> DeprecatedAwaitable:
         """Release the lock."""
         if self._owner_task != get_current_task():
-            raise RuntimeError('The current task is not holding this lock')
+            raise RuntimeError("The current task is not holding this lock")
 
         if self._waiters:
             self._owner_task, event = self._waiters.popleft()
@@ -177,23 +188,26 @@ class Lock:
 
 
 class Condition:
-    _owner_task: Optional[TaskInfo] = None
+    _owner_task: TaskInfo | None = None
 
-    def __init__(self, lock: Optional[Lock] = None):
+    def __init__(self, lock: Lock | None = None):
         self._lock = lock or Lock()
-        self._waiters: Deque[Event] = deque()
+        self._waiters: deque[Event] = deque()
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> None:
         await self.acquire()
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     def _check_acquired(self) -> None:
         if self._owner_task != get_current_task():
-            raise RuntimeError('The current task is not holding the underlying lock')
+            raise RuntimeError("The current task is not holding the underlying lock")
 
     async def acquire(self) -> None:
         """Acquire the underlying lock."""
@@ -204,7 +218,7 @@ class Condition:
         """
         Acquire the underlying lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         self._lock.acquire_nowait()
@@ -265,28 +279,33 @@ class Condition:
 
 
 class Semaphore:
-    def __init__(self, initial_value: int, *, max_value: Optional[int] = None):
+    def __init__(self, initial_value: int, *, max_value: int | None = None):
         if not isinstance(initial_value, int):
-            raise TypeError('initial_value must be an integer')
+            raise TypeError("initial_value must be an integer")
         if initial_value < 0:
-            raise ValueError('initial_value must be >= 0')
+            raise ValueError("initial_value must be >= 0")
         if max_value is not None:
             if not isinstance(max_value, int):
-                raise TypeError('max_value must be an integer or None')
+                raise TypeError("max_value must be an integer or None")
             if max_value < initial_value:
-                raise ValueError('max_value must be equal to or higher than initial_value')
+                raise ValueError(
+                    "max_value must be equal to or higher than initial_value"
+                )
 
         self._value = initial_value
         self._max_value = max_value
-        self._waiters: Deque[Event] = deque()
+        self._waiters: deque[Event] = deque()
 
-    async def __aenter__(self) -> 'Semaphore':
+    async def __aenter__(self) -> Semaphore:
         await self.acquire()
         return self
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     async def acquire(self) -> None:
@@ -302,16 +321,22 @@ class Semaphore:
             except BaseException:
                 if not event.is_set():
                     self._waiters.remove(event)
+                else:
+                    self.release()
 
                 raise
         else:
-            await cancel_shielded_checkpoint()
+            try:
+                await cancel_shielded_checkpoint()
+            except BaseException:
+                self.release()
+                raise
 
     def acquire_nowait(self) -> None:
         """
         Acquire the underlying lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         if self._value == 0:
@@ -322,7 +347,7 @@ class Semaphore:
     def release(self) -> DeprecatedAwaitable:
         """Increment the semaphore value."""
         if self._max_value is not None and self._value == self._max_value:
-            raise ValueError('semaphore released too many times')
+            raise ValueError("semaphore released too many times")
 
         if self._waiters:
             self._waiters.popleft().set()
@@ -337,7 +362,7 @@ class Semaphore:
         return self._value
 
     @property
-    def max_value(self) -> Optional[int]:
+    def max_value(self) -> int | None:
         """The maximum value of the semaphore."""
         return self._max_value
 
@@ -351,15 +376,18 @@ class Semaphore:
 
 
 class CapacityLimiter:
-    def __new__(cls, total_tokens: float):
+    def __new__(cls, total_tokens: float) -> CapacityLimiter:
         return get_asynclib().CapacityLimiter(total_tokens)
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> None:
         raise NotImplementedError
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> Optional[bool]:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
         raise NotImplementedError
 
     @property
@@ -380,9 +408,12 @@ class CapacityLimiter:
     def total_tokens(self, value: float) -> None:
         raise NotImplementedError
 
-    async def set_total_tokens(self, value) -> None:
-        warn('CapacityLimiter.set_total_tokens has been deprecated. Set the value of the'
-             '"total_tokens" attribute directly.', DeprecationWarning)
+    async def set_total_tokens(self, value: float) -> None:
+        warn(
+            "CapacityLimiter.set_total_tokens has been deprecated. Set the value of the"
+            '"total_tokens" attribute directly.',
+            DeprecationWarning,
+        )
         self.total_tokens = value
 
     @property
@@ -404,7 +435,7 @@ class CapacityLimiter:
         """
         raise NotImplementedError
 
-    def acquire_on_behalf_of_nowait(self, borrower) -> DeprecatedAwaitable:
+    def acquire_on_behalf_of_nowait(self, borrower: object) -> DeprecatedAwaitable:
         """
         Acquire a token without waiting for one to become available.
 
@@ -421,7 +452,7 @@ class CapacityLimiter:
         """
         raise NotImplementedError
 
-    async def acquire_on_behalf_of(self, borrower) -> None:
+    async def acquire_on_behalf_of(self, borrower: object) -> None:
         """
         Acquire a token, waiting if necessary for one to become available.
 
@@ -438,7 +469,7 @@ class CapacityLimiter:
         """
         raise NotImplementedError
 
-    def release_on_behalf_of(self, borrower) -> None:
+    def release_on_behalf_of(self, borrower: object) -> None:
         """
         Release the token held by the given borrower.
 
@@ -467,11 +498,11 @@ def create_lock() -> Lock:
        Use :class:`~Lock` directly.
 
     """
-    warn('create_lock() is deprecated -- use Lock() directly', DeprecationWarning)
+    warn("create_lock() is deprecated -- use Lock() directly", DeprecationWarning)
     return Lock()
 
 
-def create_condition(lock: Optional[Lock] = None) -> Condition:
+def create_condition(lock: Lock | None = None) -> Condition:
     """
     Create an asynchronous condition.
 
@@ -482,7 +513,10 @@ def create_condition(lock: Optional[Lock] = None) -> Condition:
        Use :class:`~Condition` directly.
 
     """
-    warn('create_condition() is deprecated -- use Condition() directly', DeprecationWarning)
+    warn(
+        "create_condition() is deprecated -- use Condition() directly",
+        DeprecationWarning,
+    )
     return Condition(lock=lock)
 
 
@@ -496,11 +530,11 @@ def create_event() -> Event:
        Use :class:`~Event` directly.
 
     """
-    warn('create_event() is deprecated -- use Event() directly', DeprecationWarning)
+    warn("create_event() is deprecated -- use Event() directly", DeprecationWarning)
     return get_asynclib().Event()
 
 
-def create_semaphore(value: int, *, max_value: Optional[int] = None) -> Semaphore:
+def create_semaphore(value: int, *, max_value: int | None = None) -> Semaphore:
     """
     Create an asynchronous semaphore.
 
@@ -513,7 +547,10 @@ def create_semaphore(value: int, *, max_value: Optional[int] = None) -> Semaphor
        Use :class:`~Semaphore` directly.
 
     """
-    warn('create_semaphore() is deprecated -- use Semaphore() directly', DeprecationWarning)
+    warn(
+        "create_semaphore() is deprecated -- use Semaphore() directly",
+        DeprecationWarning,
+    )
     return Semaphore(value, max_value=max_value)
 
 
@@ -529,23 +566,31 @@ def create_capacity_limiter(total_tokens: float) -> CapacityLimiter:
        Use :class:`~CapacityLimiter` directly.
 
     """
-    warn('create_capacity_limiter() is deprecated -- use CapacityLimiter() directly',
-         DeprecationWarning)
+    warn(
+        "create_capacity_limiter() is deprecated -- use CapacityLimiter() directly",
+        DeprecationWarning,
+    )
     return get_asynclib().CapacityLimiter(total_tokens)
 
 
 class ResourceGuard:
-    __slots__ = 'action', '_guarded'
+    __slots__ = "action", "_guarded"
 
     def __init__(self, action: str):
         self.action = action
         self._guarded = False
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         if self._guarded:
             raise BusyResourceError(self.action)
 
         self._guarded = True
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
         self._guarded = False
+        return None
