@@ -1,11 +1,23 @@
+from __future__ import annotations
+
 import threading
 from asyncio import iscoroutine
 from concurrent.futures import FIRST_COMPLETED, Future, ThreadPoolExecutor, wait
 from contextlib import AbstractContextManager, contextmanager
 from types import TracebackType
 from typing import (
-    Any, AsyncContextManager, Callable, ContextManager, Coroutine, Dict, Generator, Iterable,
-    Optional, Tuple, Type, TypeVar, Union, cast, overload)
+    Any,
+    AsyncContextManager,
+    Awaitable,
+    Callable,
+    ContextManager,
+    Generator,
+    Generic,
+    Iterable,
+    TypeVar,
+    cast,
+    overload,
+)
 from warnings import warn
 
 from ._core import _eventloop
@@ -14,11 +26,11 @@ from ._core._synchronization import Event
 from ._core._tasks import CancelScope, create_task_group
 from .abc._tasks import TaskStatus
 
-T_Retval = TypeVar('T_Retval')
-T_co = TypeVar('T_co')
+T_Retval = TypeVar("T_Retval")
+T_co = TypeVar("T_co")
 
 
-def run(func: Callable[..., Coroutine[Any, Any, T_Retval]], *args: object) -> T_Retval:
+def run(func: Callable[..., Awaitable[T_Retval]], *args: object) -> T_Retval:
     """
     Call a coroutine function from a worker thread.
 
@@ -30,15 +42,18 @@ def run(func: Callable[..., Coroutine[Any, Any, T_Retval]], *args: object) -> T_
     try:
         asynclib = threadlocals.current_async_module
     except AttributeError:
-        raise RuntimeError('This function can only be run from an AnyIO worker thread')
+        raise RuntimeError("This function can only be run from an AnyIO worker thread")
 
     return asynclib.run_async_from_thread(func, *args)
 
 
-def run_async_from_thread(func: Callable[..., Coroutine[Any, Any, T_Retval]],
-                          *args: object) -> T_Retval:
-    warn('run_async_from_thread() has been deprecated, use anyio.from_thread.run() instead',
-         DeprecationWarning)
+def run_async_from_thread(
+    func: Callable[..., Awaitable[T_Retval]], *args: object
+) -> T_Retval:
+    warn(
+        "run_async_from_thread() has been deprecated, use anyio.from_thread.run() instead",
+        DeprecationWarning,
+    )
     return run(func, *args)
 
 
@@ -54,29 +69,32 @@ def run_sync(func: Callable[..., T_Retval], *args: object) -> T_Retval:
     try:
         asynclib = threadlocals.current_async_module
     except AttributeError:
-        raise RuntimeError('This function can only be run from an AnyIO worker thread')
+        raise RuntimeError("This function can only be run from an AnyIO worker thread")
 
     return asynclib.run_sync_from_thread(func, *args)
 
 
 def run_sync_from_thread(func: Callable[..., T_Retval], *args: object) -> T_Retval:
-    warn('run_sync_from_thread() has been deprecated, use anyio.from_thread.run_sync() instead',
-         DeprecationWarning)
+    warn(
+        "run_sync_from_thread() has been deprecated, use anyio.from_thread.run_sync() instead",
+        DeprecationWarning,
+    )
     return run_sync(func, *args)
 
 
-class _BlockingAsyncContextManager(AbstractContextManager):
+class _BlockingAsyncContextManager(Generic[T_co], AbstractContextManager):
     _enter_future: Future
     _exit_future: Future
     _exit_event: Event
-    _exit_exc_info: Tuple[Optional[Type[BaseException]], Optional[BaseException],
-                          Optional[TracebackType]] = (None, None, None)
+    _exit_exc_info: tuple[
+        type[BaseException] | None, BaseException | None, TracebackType | None
+    ] = (None, None, None)
 
-    def __init__(self, async_cm: AsyncContextManager[T_co], portal: 'BlockingPortal'):
+    def __init__(self, async_cm: AsyncContextManager[T_co], portal: BlockingPortal):
         self._async_cm = async_cm
         self._portal = portal
 
-    async def run_async_cm(self) -> Optional[bool]:
+    async def run_async_cm(self) -> bool | None:
         try:
             self._exit_event = Event()
             value = await self._async_cm.__aenter__()
@@ -105,9 +123,12 @@ class _BlockingAsyncContextManager(AbstractContextManager):
         cm = self._enter_future.result()
         return cast(T_co, cm)
 
-    def __exit__(self, __exc_type: Optional[Type[BaseException]],
-                 __exc_value: Optional[BaseException],
-                 __traceback: Optional[TracebackType]) -> Optional[bool]:
+    def __exit__(
+        self,
+        __exc_type: type[BaseException] | None,
+        __exc_value: BaseException | None,
+        __traceback: TracebackType | None,
+    ) -> bool | None:
         self._exit_exc_info = __exc_type, __exc_value, __traceback
         self._portal.call(self._exit_event.set)
         return self._exit_future.result()
@@ -124,30 +145,35 @@ class _BlockingPortalTaskStatus(TaskStatus):
 class BlockingPortal:
     """An object that lets external threads run code in an asynchronous event loop."""
 
-    def __new__(cls) -> 'BlockingPortal':
+    def __new__(cls) -> BlockingPortal:
         return get_asynclib().BlockingPortal()
 
     def __init__(self) -> None:
-        self._event_loop_thread_id: Optional[int] = threading.get_ident()
+        self._event_loop_thread_id: int | None = threading.get_ident()
         self._stop_event = Event()
         self._task_group = create_task_group()
         self._cancelled_exc_class = get_cancelled_exc_class()
 
-    async def __aenter__(self) -> 'BlockingPortal':
+    async def __aenter__(self) -> BlockingPortal:
         await self._task_group.__aenter__()
         return self
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> Optional[bool]:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
         await self.stop()
         return await self._task_group.__aexit__(exc_type, exc_val, exc_tb)
 
     def _check_running(self) -> None:
         if self._event_loop_thread_id is None:
-            raise RuntimeError('This portal is not running')
+            raise RuntimeError("This portal is not running")
         if self._event_loop_thread_id == threading.get_ident():
-            raise RuntimeError('This method cannot be called from the event loop thread')
+            raise RuntimeError(
+                "This method cannot be called from the event loop thread"
+            )
 
     async def sleep_until_stopped(self) -> None:
         """Sleep until :meth:`stop` is called."""
@@ -169,10 +195,14 @@ class BlockingPortal:
         if cancel_remaining:
             self._task_group.cancel_scope.cancel()
 
-    async def _call_func(self, func: Callable, args: tuple, kwargs: Dict[str, Any],
-                         future: Future) -> None:
+    async def _call_func(
+        self, func: Callable, args: tuple, kwargs: dict[str, Any], future: Future
+    ) -> None:
         def callback(f: Future) -> None:
-            if f.cancelled() and self._event_loop_thread_id not in (None, threading.get_ident()):
+            if f.cancelled() and self._event_loop_thread_id not in (
+                None,
+                threading.get_ident(),
+            ):
                 self.call(scope.cancel)
 
         try:
@@ -200,8 +230,14 @@ class BlockingPortal:
         finally:
             scope = None  # type: ignore[assignment]
 
-    def _spawn_task_from_thread(self, func: Callable, args: tuple, kwargs: Dict[str, Any],
-                                name: object, future: Future) -> None:
+    def _spawn_task_from_thread(
+        self,
+        func: Callable,
+        args: tuple,
+        kwargs: dict[str, Any],
+        name: object,
+        future: Future,
+    ) -> None:
         """
         Spawn a new task using the given callable.
 
@@ -218,15 +254,16 @@ class BlockingPortal:
         raise NotImplementedError
 
     @overload
-    def call(self, func: Callable[..., Coroutine[Any, Any, T_Retval]], *args: object) -> T_Retval:
+    def call(self, func: Callable[..., Awaitable[T_Retval]], *args: object) -> T_Retval:
         ...
 
     @overload
     def call(self, func: Callable[..., T_Retval], *args: object) -> T_Retval:
         ...
 
-    def call(self, func: Callable[..., Union[Coroutine[Any, Any, T_Retval], T_Retval]],
-             *args: object) -> T_Retval:
+    def call(
+        self, func: Callable[..., Awaitable[T_Retval] | T_Retval], *args: object
+    ) -> T_Retval:
         """
         Call the given function in the event loop thread.
 
@@ -240,16 +277,26 @@ class BlockingPortal:
         return cast(T_Retval, self.start_task_soon(func, *args).result())
 
     @overload
-    def spawn_task(self, func: Callable[..., Coroutine[Any, Any, T_Retval]],
-                   *args: object, name: object = None) -> "Future[T_Retval]":
+    def spawn_task(
+        self,
+        func: Callable[..., Awaitable[T_Retval]],
+        *args: object,
+        name: object = None,
+    ) -> Future[T_Retval]:
         ...
 
     @overload
-    def spawn_task(self, func: Callable[..., T_Retval],
-                   *args: object, name: object = None) -> "Future[T_Retval]": ...
+    def spawn_task(
+        self, func: Callable[..., T_Retval], *args: object, name: object = None
+    ) -> Future[T_Retval]:
+        ...
 
-    def spawn_task(self, func: Callable[..., Union[Coroutine[Any, Any, T_Retval], T_Retval]],
-                   *args: object, name: object = None) -> "Future[T_Retval]":
+    def spawn_task(
+        self,
+        func: Callable[..., Awaitable[T_Retval] | T_Retval],
+        *args: object,
+        name: object = None,
+    ) -> Future[T_Retval]:
         """
         Start a task in the portal's task group.
 
@@ -267,33 +314,47 @@ class BlockingPortal:
            can keep using this until AnyIO 4.
 
         """
-        warn('spawn_task() is deprecated -- use start_task_soon() instead', DeprecationWarning)
+        warn(
+            "spawn_task() is deprecated -- use start_task_soon() instead",
+            DeprecationWarning,
+        )
         return self.start_task_soon(func, *args, name=name)  # type: ignore[arg-type]
 
     @overload
-    def start_task_soon(self, func: Callable[..., Coroutine[Any, Any, T_Retval]],
-                        *args: object, name: object = None) -> "Future[T_Retval]":
+    def start_task_soon(
+        self,
+        func: Callable[..., Awaitable[T_Retval]],
+        *args: object,
+        name: object = None,
+    ) -> Future[T_Retval]:
         ...
 
     @overload
-    def start_task_soon(self, func: Callable[..., T_Retval],
-                        *args: object, name: object = None) -> "Future[T_Retval]": ...
+    def start_task_soon(
+        self, func: Callable[..., T_Retval], *args: object, name: object = None
+    ) -> Future[T_Retval]:
+        ...
 
-    def start_task_soon(self, func: Callable[..., Union[Coroutine[Any, Any, T_Retval], T_Retval]],
-                        *args: object, name: object = None) -> "Future[T_Retval]":
+    def start_task_soon(
+        self,
+        func: Callable[..., Awaitable[T_Retval] | T_Retval],
+        *args: object,
+        name: object = None,
+    ) -> Future[T_Retval]:
         """
         Start a task in the portal's task group.
 
         The task will be run inside a cancel scope which can be cancelled by cancelling the
         returned future.
 
-        :param func: the target coroutine function
+        :param func: the target function
         :param args: positional arguments passed to ``func``
         :param name: name of the task (will be coerced to a string if not ``None``)
-        :return: a future that resolves with the return value of the callable if the task completes
-            successfully, or with the exception raised in the task
-        :raises RuntimeError: if the portal is not running or if this method is called from within
-            the event loop thread
+        :return: a future that resolves with the return value of the callable if the
+            task completes successfully, or with the exception raised in the task
+        :raises RuntimeError: if the portal is not running or if this method is called
+            from within the event loop thread
+        :rtype: concurrent.futures.Future[T_Retval]
 
         .. versionadded:: 3.0
 
@@ -303,22 +364,26 @@ class BlockingPortal:
         self._spawn_task_from_thread(func, args, {}, name, f)
         return f
 
-    def start_task(self, func: Callable[..., Coroutine], *args: object,
-                   name: object = None) -> Tuple[Future, Any]:
+    def start_task(
+        self, func: Callable[..., Awaitable[Any]], *args: object, name: object = None
+    ) -> tuple[Future[Any], Any]:
         """
         Start a task in the portal's task group and wait until it signals for readiness.
 
-        This method works the same way as :meth:`TaskGroup.start`.
+        This method works the same way as :meth:`.abc.TaskGroup.start`.
 
-        :param func: the target coroutine function
+        :param func: the target function
         :param args: positional arguments passed to ``func``
         :param name: name of the task (will be coerced to a string if not ``None``)
-        :return: a tuple of (future, task_status_value) where the ``task_status_value`` is the
-            value passed to ``task_status.started()`` from within the target function
+        :return: a tuple of (future, task_status_value) where the ``task_status_value``
+            is the value passed to ``task_status.started()`` from within the target
+            function
+        :rtype: tuple[concurrent.futures.Future[Any], Any]
 
         .. versionadded:: 3.0
 
         """
+
         def task_done(future: Future) -> None:
             if not task_status_future.done():
                 if future.cancelled():
@@ -326,7 +391,9 @@ class BlockingPortal:
                 elif future.exception():
                     task_status_future.set_exception(future.exception())
                 else:
-                    exc = RuntimeError('Task exited without calling task_status.started()')
+                    exc = RuntimeError(
+                        "Task exited without calling task_status.started()"
+                    )
                     task_status_future.set_exception(exc)
 
         self._check_running()
@@ -334,10 +401,12 @@ class BlockingPortal:
         task_status = _BlockingPortalTaskStatus(task_status_future)
         f: Future = Future()
         f.add_done_callback(task_done)
-        self._spawn_task_from_thread(func, args, {'task_status': task_status}, name, f)
+        self._spawn_task_from_thread(func, args, {"task_status": task_status}, name, f)
         return f, task_status_future.result()
 
-    def wrap_async_context_manager(self, cm: AsyncContextManager[T_co]) -> ContextManager[T_co]:
+    def wrap_async_context_manager(
+        self, cm: AsyncContextManager[T_co]
+    ) -> ContextManager[T_co]:
         """
         Wrap an async context manager as a synchronous context manager via this portal.
 
@@ -364,15 +433,18 @@ def create_blocking_portal() -> BlockingPortal:
         Use :class:`.BlockingPortal` directly.
 
     """
-    warn('create_blocking_portal() has been deprecated -- use anyio.from_thread.BlockingPortal() '
-         'directly', DeprecationWarning)
+    warn(
+        "create_blocking_portal() has been deprecated -- use anyio.from_thread.BlockingPortal() "
+        "directly",
+        DeprecationWarning,
+    )
     return BlockingPortal()
 
 
 @contextmanager
 def start_blocking_portal(
-        backend: str = 'asyncio',
-        backend_options: Optional[Dict[str, Any]] = None) -> Generator[BlockingPortal, Any, None]:
+    backend: str = "asyncio", backend_options: dict[str, Any] | None = None
+) -> Generator[BlockingPortal, Any, None]:
     """
     Start a new event loop in a new thread and run a blocking portal in its main task.
 
@@ -386,6 +458,7 @@ def start_blocking_portal(
         Usage as a context manager is now required.
 
     """
+
     async def run_portal() -> None:
         async with BlockingPortal() as portal_:
             if future.set_running_or_notify_cancel():
@@ -394,10 +467,17 @@ def start_blocking_portal(
 
     future: Future[BlockingPortal] = Future()
     with ThreadPoolExecutor(1) as executor:
-        run_future = executor.submit(_eventloop.run, run_portal, backend=backend,
-                                     backend_options=backend_options)
+        run_future = executor.submit(
+            _eventloop.run,
+            run_portal,  # type: ignore[arg-type]
+            backend=backend,
+            backend_options=backend_options,
+        )
         try:
-            wait(cast(Iterable[Future], [run_future, future]), return_when=FIRST_COMPLETED)
+            wait(
+                cast(Iterable[Future], [run_future, future]),
+                return_when=FIRST_COMPLETED,
+            )
         except BaseException:
             future.cancel()
             run_future.cancel()
@@ -405,12 +485,16 @@ def start_blocking_portal(
 
         if future.done():
             portal = future.result()
+            cancel_remaining_tasks = False
             try:
                 yield portal
             except BaseException:
-                portal.call(portal.stop, True)
+                cancel_remaining_tasks = True
                 raise
-
-            portal.call(portal.stop, False)
+            finally:
+                try:
+                    portal.call(portal.stop, cancel_remaining_tasks)
+                except RuntimeError:
+                    pass
 
         run_future.result()
