@@ -1,24 +1,39 @@
+from __future__ import annotations
+
 import asyncio
+import math
 import re
 import sys
 import time
-from typing import Any, AsyncGenerator, Coroutine, Dict, Generator, NoReturn, Optional, Set
+from typing import (
+    Any,
+    AsyncGenerator,
+    Coroutine,
+    Generator,
+    NoReturn,
+    cast,
+)
 
 import pytest
 
 import anyio
 from anyio import (
-    CancelScope, ExceptionGroup, create_task_group, current_effective_deadline, current_time,
-    fail_after, get_cancelled_exc_class, get_current_task, move_on_after, sleep,
-    wait_all_tasks_blocked)
+    TASK_STATUS_IGNORED,
+    CancelScope,
+    ExceptionGroup,
+    create_task_group,
+    current_effective_deadline,
+    current_time,
+    fail_after,
+    get_cancelled_exc_class,
+    get_current_task,
+    move_on_after,
+    sleep,
+    wait_all_tasks_blocked,
+)
 from anyio.abc import TaskGroup, TaskStatus
 from anyio.lowlevel import checkpoint
 
-if sys.version_info < (3, 7):
-    current_task = asyncio.Task.current_task
-else:
-    current_task = asyncio.current_task
-
 pytestmark = pytest.mark.anyio
 
 
@@ -35,27 +50,30 @@ async def test_already_closed() -> None:
         pass
 
     with pytest.raises(RuntimeError) as exc:
-        tg.start_soon(async_error, 'fail')
+        tg.start_soon(async_error, "fail")
 
-    exc.match('This task group is not active; no new tasks can be started')
+    exc.match("This task group is not active; no new tasks can be started")
 
 
 async def test_success() -> None:
     async def async_add(value: str) -> None:
         results.add(value)
 
-    results: Set[str] = set()
+    results: set[str] = set()
     async with create_task_group() as tg:
-        tg.start_soon(async_add, 'a')
-        tg.start_soon(async_add, 'b')
+        tg.start_soon(async_add, "a")
+        tg.start_soon(async_add, "b")
 
-    assert results == {'a', 'b'}
+    assert results == {"a", "b"}
 
 
-@pytest.mark.parametrize('module', [
-    pytest.param(asyncio, id='asyncio'),
-    pytest.param(pytest.importorskip('trio'), id='trio')
-])
+@pytest.mark.parametrize(
+    "module",
+    [
+        pytest.param(asyncio, id="asyncio"),
+        pytest.param(pytest.importorskip("trio"), id="trio"),
+    ],
+)
 def test_run_natively(module: Any) -> None:
     async def testfunc() -> None:
         async with create_task_group() as tg:
@@ -63,6 +81,7 @@ def test_run_natively(module: Any) -> None:
 
     if module is asyncio:
         from anyio._backends._asyncio import native_run  # type: ignore[attr-defined]
+
         try:
             native_run(testfunc())
         finally:
@@ -87,7 +106,7 @@ async def test_start_soon_after_error() -> None:
     with pytest.raises(RuntimeError) as exc:
         tg.start_soon(sleep, 0)
 
-    exc.match('This task group is not active; no new tasks can be started')
+    exc.match("This task group is not active; no new tasks can be started")
 
 
 async def test_start_no_value() -> None:
@@ -103,8 +122,9 @@ async def test_start_called_twice() -> None:
     async def taskfunc(*, task_status: TaskStatus) -> None:
         task_status.started()
 
-        with pytest.raises(RuntimeError,
-                           match="called 'started' twice on the same task status"):
+        with pytest.raises(
+            RuntimeError, match="called 'started' twice on the same task status"
+        ):
             task_status.started()
 
     async with create_task_group() as tg:
@@ -114,34 +134,34 @@ async def test_start_called_twice() -> None:
 
 async def test_start_with_value() -> None:
     async def taskfunc(*, task_status: TaskStatus) -> None:
-        task_status.started('foo')
+        task_status.started("foo")
 
     async with create_task_group() as tg:
         value = await tg.start(taskfunc)
-        assert value == 'foo'
+        assert value == "foo"
 
 
 async def test_start_crash_before_started_call() -> None:
     async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
-        raise Exception('foo')
+        raise Exception("foo")
 
     async with create_task_group() as tg:
         with pytest.raises(Exception) as exc:
             await tg.start(taskfunc)
 
-    exc.match('foo')
+    exc.match("foo")
 
 
 async def test_start_crash_after_started_call() -> None:
     async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
         task_status.started(2)
-        raise Exception('foo')
+        raise Exception("foo")
 
     with pytest.raises(Exception) as exc:
         async with create_task_group() as tg:
             value = await tg.start(taskfunc)
 
-    exc.match('foo')
+    exc.match("foo")
     assert value == 2
 
 
@@ -153,7 +173,7 @@ async def test_start_no_started_call() -> None:
         with pytest.raises(RuntimeError) as exc:
             await tg.start(taskfunc)
 
-    exc.match('hild exited')
+    exc.match("hild exited")
 
 
 async def test_start_cancelled() -> None:
@@ -173,7 +193,7 @@ async def test_start_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_start_native_host_cancelled() -> None:
     started = finished = False
 
@@ -197,14 +217,14 @@ async def test_start_native_host_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_start_native_child_cancelled() -> None:
     task = None
     finished = False
 
     async def taskfunc(*, task_status: TaskStatus) -> None:
         nonlocal task, finished
-        task = current_task()
+        task = asyncio.current_task()
         await sleep(2)
         finished = True
 
@@ -226,7 +246,7 @@ async def test_start_exception_delivery() -> None:
         task_status.started("hello")
 
     async with anyio.create_task_group() as tg:
-        with pytest.raises(TypeError, match='to be synchronous$'):
+        with pytest.raises(TypeError, match="to be synchronous$"):
             await tg.start(task_fn)  # type: ignore[arg-type]
 
 
@@ -240,10 +260,10 @@ async def test_host_exception() -> None:
 
     with pytest.raises(Exception) as exc:
         async with create_task_group() as tg:
-            tg.start_soon(set_result, 'a')
-            raise Exception('dummy error')
+            tg.start_soon(set_result, "a")
+            raise Exception("dummy error")
 
-    exc.match('dummy error')
+    exc.match("dummy error")
     assert result is None
 
 
@@ -269,7 +289,7 @@ async def test_level_cancellation() -> None:
 async def test_failing_child_task_cancels_host() -> None:
     async def child() -> NoReturn:
         await wait_all_tasks_blocked()
-        raise Exception('foo')
+        raise Exception("foo")
 
     sleep_completed = False
     with pytest.raises(Exception) as exc:
@@ -278,7 +298,7 @@ async def test_failing_child_task_cancels_host() -> None:
             await sleep(0.5)
             sleep_completed = True
 
-    exc.match('foo')
+    exc.match("foo")
     assert not sleep_completed
 
 
@@ -294,9 +314,9 @@ async def test_failing_host_task_cancels_children() -> None:
         async with create_task_group() as tg:
             tg.start_soon(child)
             await wait_all_tasks_blocked()
-            raise Exception('foo')
+            raise Exception("foo")
 
-    exc.match('foo')
+    exc.match("foo")
     assert not sleep_completed
 
 
@@ -335,6 +355,7 @@ async def test_cancel_propagation() -> None:
 
 async def test_cancel_twice() -> None:
     """Test that the same task can receive two cancellations."""
+
     async def cancel_group() -> None:
         await wait_all_tasks_blocked()
         tg.cancel_scope.cancel()
@@ -343,7 +364,7 @@ async def test_cancel_twice() -> None:
         async with create_task_group() as tg:
             tg.start_soon(cancel_group)
             await sleep(1)
-            pytest.fail('Execution should not reach this point')
+            pytest.fail("Execution should not reach this point")
 
 
 async def test_cancel_exiting_task_group() -> None:
@@ -373,32 +394,45 @@ async def test_cancel_exiting_task_group() -> None:
     assert cancel_received
 
 
+async def test_cancel_before_entering_scope() -> None:
+    """
+    Test that CancelScope.cancel() is honored even if called before entering the scope.
+
+    """
+    cancel_scope = anyio.CancelScope()
+    cancel_scope.cancel()
+    with cancel_scope:
+        await anyio.sleep(1)  # Checkpoint to allow anyio to check for cancellation
+        pytest.fail("execution should not reach this point")
+
+
 async def test_exception_group_children() -> None:
     with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
-            tg.start_soon(async_error, 'task1')
-            tg.start_soon(async_error, 'task2', 0.15)
+            tg.start_soon(async_error, "task1")
+            tg.start_soon(async_error, "task2", 0.15)
 
     assert len(exc.value.exceptions) == 2
-    assert sorted(str(e) for e in exc.value.exceptions) == ['task1', 'task2']
-    assert exc.match('^2 exceptions were raised in the task group:\n')
-    assert exc.match(r'Exception: task\d\n----')
+    assert sorted(str(e) for e in exc.value.exceptions) == ["task1", "task2"]
+    assert exc.match("^2 exceptions were raised in the task group:\n")
+    assert exc.match(r"Exception: task\d\n----")
     assert re.fullmatch(
         r"<ExceptionGroup: Exception\('task[12]',?\), Exception\('task[12]',?\)>",
-        repr(exc.value))
+        repr(exc.value),
+    )
 
 
 async def test_exception_group_host() -> None:
     with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
-            tg.start_soon(async_error, 'child', 2)
+            tg.start_soon(async_error, "child", 2)
             await wait_all_tasks_blocked()
-            raise Exception('host')
+            raise Exception("host")
 
     assert len(exc.value.exceptions) == 2
-    assert sorted(str(e) for e in exc.value.exceptions) == ['child', 'host']
-    assert exc.match('^2 exceptions were raised in the task group:\n')
-    assert exc.match(r'Exception: host\n----')
+    assert sorted(str(e) for e in exc.value.exceptions) == ["child", "host"]
+    assert exc.match("^2 exceptions were raised in the task group:\n")
+    assert exc.match(r"Exception: host\n----")
 
 
 async def test_escaping_cancelled_exception() -> None:
@@ -414,7 +448,7 @@ async def test_cancel_scope_cleared() -> None:
     await sleep(0)
 
 
-@pytest.mark.parametrize('delay', [0, 0.1], ids=['instant', 'delayed'])
+@pytest.mark.parametrize("delay", [0, 0.1], ids=["instant", "delayed"])
 async def test_fail_after(delay: float) -> None:
     with pytest.raises(TimeoutError):
         with fail_after(delay) as scope:
@@ -425,7 +459,7 @@ async def test_fail_after(delay: float) -> None:
 
 async def test_fail_after_no_timeout() -> None:
     with fail_after(None) as scope:
-        assert scope.deadline == float('inf')
+        assert scope.deadline == float("inf")
         await sleep(0.1)
 
     assert not scope.cancel_called
@@ -446,7 +480,7 @@ async def test_fail_after_after_cancellation() -> None:
     assert not block_complete
 
 
-@pytest.mark.parametrize('delay', [0, 0.1], ids=['instant', 'delayed'])
+@pytest.mark.parametrize("delay", [0, 0.1], ids=["instant", "delayed"])
 async def test_move_on_after(delay: float) -> None:
     result = False
     with move_on_after(delay) as scope:
@@ -460,7 +494,7 @@ async def test_move_on_after(delay: float) -> None:
 async def test_move_on_after_no_timeout() -> None:
     result = False
     with move_on_after(None) as scope:
-        assert scope.deadline == float('inf')
+        assert scope.deadline == float("inf")
         await sleep(0.1)
         result = True
 
@@ -512,6 +546,9 @@ async def test_cancel_from_shielded_scope() -> None:
         with CancelScope(shield=True) as inner_scope:
             assert inner_scope.shield
             tg.cancel_scope.cancel()
+            assert current_effective_deadline() == math.inf
+
+        assert current_effective_deadline() == -math.inf
 
         with pytest.raises(get_cancelled_exc_class()):
             await sleep(0.01)
@@ -520,7 +557,17 @@ async def test_cancel_from_shielded_scope() -> None:
             await sleep(0.01)
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+async def test_cancel_shielded_scope() -> None:
+    with CancelScope(shield=True) as cancel_scope:
+        assert cancel_scope.shield
+        cancel_scope.cancel()
+        assert current_effective_deadline() == -math.inf
+
+        with pytest.raises(get_cancelled_exc_class()):
+            await sleep(0)
+
+
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_host_asyncgen() -> None:
     done = False
 
@@ -546,7 +593,7 @@ async def test_cancel_host_asyncgen() -> None:
 
     host_agen = host_agen_fn()
     try:
-        await asyncio.get_event_loop().create_task(host_agen.__anext__())
+        await asyncio.get_event_loop().create_task(host_agen.__anext__())  # type: ignore[arg-type]
     finally:
         await host_agen.aclose()
 
@@ -571,19 +618,19 @@ async def test_shielding_immediate_scope_cancelled() -> None:
 async def test_shielding_mutate() -> None:
     completed = False
 
-    async def task(task_status: TaskStatus) -> NoReturn:  # type: ignore[misc]
+    async def task(task_status: TaskStatus) -> NoReturn:
         nonlocal completed
         with CancelScope() as scope:
             # Enable the shield a little after the scope starts to make this test
             # general, even though it has no bearing on the current implementation.
-            await sleep(.1)
+            await sleep(0.1)
             scope.shield = True
             task_status.started()
-            await sleep(.1)
+            await sleep(0.1)
             completed = True
             scope.shield = False
             await sleep(1)
-            pytest.fail('Execution should not reach this point')
+            pytest.fail("Execution should not reach this point")
 
     async with create_task_group() as tg:
         await tg.start(task)
@@ -618,7 +665,7 @@ async def test_exception_cancels_siblings() -> None:
 
     async def child(fail: bool) -> None:
         if fail:
-            raise Exception('foo')
+            raise Exception("foo")
         else:
             nonlocal sleep_completed
             await sleep(1)
@@ -630,7 +677,7 @@ async def test_exception_cancels_siblings() -> None:
             await wait_all_tasks_blocked()
             tg.start_soon(child, True)
 
-    exc.match('foo')
+    exc.match("foo")
     assert not sleep_completed
 
 
@@ -639,7 +686,7 @@ async def test_cancel_cascade() -> None:
         async with create_task_group() as tg2:
             tg2.start_soon(sleep, 1)
 
-        raise Exception('foo')
+        raise Exception("foo")
 
     async with create_task_group() as tg:
         tg.start_soon(do_something)
@@ -652,7 +699,7 @@ async def test_cancelled_parent() -> None:
         with CancelScope():
             await sleep(1)
 
-        raise Exception('foo')
+        raise Exception("foo")
 
     async def parent(tg: TaskGroup) -> None:
         await wait_all_tasks_blocked()
@@ -673,7 +720,7 @@ async def test_shielded_deadline() -> None:
 async def test_deadline_reached_on_start() -> None:
     with move_on_after(0):
         await sleep(0)
-        pytest.fail('Execution should not reach this point')
+        pytest.fail("Execution should not reach this point")
 
 
 async def test_deadline_moved() -> None:
@@ -701,11 +748,11 @@ async def test_nested_fail_after() -> None:
                 tg.start_soon(killer, scope)
                 with fail_after(1):
                     await sleep(2)
-                    pytest.fail('Execution should not reach this point')
+                    pytest.fail("Execution should not reach this point")
 
-                pytest.fail('Execution should not reach this point either')
+                pytest.fail("Execution should not reach this point either")
 
-            pytest.fail('Execution should also not reach this point')
+            pytest.fail("Execution should also not reach this point")
 
     assert scope.cancel_called
 
@@ -747,17 +794,26 @@ async def test_triple_nested_shield() -> None:
     assert not got_past_checkpoint
 
 
-def test_task_group_in_generator(anyio_backend_name: str,
-                                 anyio_backend_options: Dict[str, Any]) -> None:
+def test_task_group_in_generator(
+    anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+) -> None:
     async def task_group_generator() -> AsyncGenerator[None, None]:
         async with create_task_group():
             yield
 
     gen = task_group_generator()
-    anyio.run(gen.__anext__, backend=anyio_backend_name,  # type: ignore[arg-type]
-              backend_options=anyio_backend_options)
-    pytest.raises(StopAsyncIteration, anyio.run, gen.__anext__, backend=anyio_backend_name,
-                  backend_options=anyio_backend_options)
+    anyio.run(
+        gen.__anext__,
+        backend=anyio_backend_name,
+        backend_options=anyio_backend_options,
+    )
+    pytest.raises(
+        StopAsyncIteration,
+        anyio.run,
+        gen.__anext__,
+        backend=anyio_backend_name,
+        backend_options=anyio_backend_options,
+    )
 
 
 async def test_exception_group_filtering() -> None:
@@ -765,23 +821,23 @@ async def test_exception_group_filtering() -> None:
 
     async def fail(name: str) -> NoReturn:
         try:
-            await anyio.sleep(.1)
+            await anyio.sleep(0.1)
         finally:
-            raise Exception('%s task failed' % name)
+            raise Exception("%s task failed" % name)
 
     async def fn() -> None:
         async with anyio.create_task_group() as tg:
-            tg.start_soon(fail, 'parent')
+            tg.start_soon(fail, "parent")
             async with anyio.create_task_group() as tg2:
-                tg2.start_soon(fail, 'child')
+                tg2.start_soon(fail, "child")
                 await anyio.sleep(1)
 
     with pytest.raises(ExceptionGroup) as exc:
         await fn()
 
     assert len(exc.value.exceptions) == 2
-    assert str(exc.value.exceptions[0]) == 'parent task failed'
-    assert str(exc.value.exceptions[1]) == 'child task failed'
+    assert str(exc.value.exceptions[0]) == "parent task failed"
+    assert str(exc.value.exceptions[1]) == "child task failed"
 
 
 async def test_cancel_propagation_with_inner_spawn() -> None:
@@ -807,9 +863,15 @@ async def test_escaping_cancelled_error_from_cancelled_task() -> None:
         scope.cancel()
 
 
-@pytest.mark.filterwarnings('ignore:"@coroutine" decorator is deprecated:DeprecationWarning')
+@pytest.mark.skipif(
+    sys.version_info >= (3, 11),
+    reason="Generator based coroutines have been removed in Python 3.11",
+)
+@pytest.mark.filterwarnings(
+    'ignore:"@coroutine" decorator is deprecated:DeprecationWarning'
+)
 def test_cancel_generator_based_task() -> None:
-    from asyncio import coroutine
+    from asyncio import coroutine  # type: ignore[attr-defined]
 
     async def native_coro_part() -> None:
         with CancelScope() as scope:
@@ -817,9 +879,9 @@ def test_cancel_generator_based_task() -> None:
 
     @coroutine
     def generator_part() -> Generator[object, BaseException, None]:
-        yield from native_coro_part()
+        yield from native_coro_part()  # type: ignore[misc]
 
-    anyio.run(generator_part, backend='asyncio')  # type: ignore[arg-type]
+    anyio.run(generator_part, backend="asyncio")
 
 
 async def test_suppress_exception_context() -> None:
@@ -838,7 +900,7 @@ async def test_suppress_exception_context() -> None:
     assert exc.value.__context__ is None
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_native_future_tasks() -> None:
     async def wait_native_future() -> None:
         loop = asyncio.get_event_loop()
@@ -849,7 +911,7 @@ async def test_cancel_native_future_tasks() -> None:
         tg.cancel_scope.cancel()
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_native_future_tasks_cancel_scope() -> None:
     async def wait_native_future() -> None:
         with anyio.CancelScope():
@@ -861,7 +923,7 @@ async def test_cancel_native_future_tasks_cancel_scope() -> None:
         tg.cancel_scope.cancel()
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_completed_task() -> None:
     loop = asyncio.get_event_loop()
     old_exception_handler = loop.get_exception_handler()
@@ -872,6 +934,7 @@ async def test_cancel_completed_task() -> None:
 
     loop.set_exception_handler(exception_handler)
     try:
+
         async def noop() -> None:
             pass
 
@@ -947,7 +1010,9 @@ async def test_cancelscope_exit_before_enter() -> None:
     pytest.raises(RuntimeError, scope.__exit__, None, None, None)
 
 
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])  # trio does not check for this yet
+@pytest.mark.parametrize(
+    "anyio_backend", ["asyncio"]
+)  # trio does not check for this yet
 async def test_cancelscope_exit_in_wrong_task() -> None:
     async def enter_scope(scope: CancelScope) -> None:
         scope.__enter__()
@@ -973,42 +1038,45 @@ def test_unhandled_exception_group(caplog: pytest.LogCaptureFixture) -> None:
             tg.start_soon(anyio.sleep, 5)
             await anyio.sleep(5)
 
-    async def main() -> NoReturn:  # type: ignore[misc]
+    async def main() -> NoReturn:
         async with anyio.create_task_group() as tg:
             tg.start_soon(nested)
             await wait_all_tasks_blocked()
             asyncio.get_event_loop().call_soon(crash)
             await anyio.sleep(5)
 
-        pytest.fail('Execution should never reach this point')
+        pytest.fail("Execution should never reach this point")
 
     with pytest.raises(KeyboardInterrupt):
-        anyio.run(main, backend='asyncio')
+        anyio.run(main, backend="asyncio")
 
     assert not caplog.messages
 
 
-@pytest.mark.skipif(sys.version_info < (3, 9),
-                    reason='Cancel messages are only supported on py3.9+')
-@pytest.mark.parametrize('anyio_backend', ['asyncio'])
+@pytest.mark.skipif(
+    sys.version_info < (3, 9),
+    sys.version_info >= (3, 11),
+    reason="Cancel messages are only supported on Python 3.9 and 3.10",
+)
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancellederror_combination_with_message() -> None:
-    async def taskfunc(*, task_status: TaskStatus) -> NoReturn:  # type: ignore[misc]
+    async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
         task_status.started(asyncio.current_task())
         await sleep(5)
-        pytest.fail('Execution should never reach this point')
+        pytest.fail("Execution should never reach this point")
 
-    with pytest.raises(asyncio.CancelledError, match='blah'):
+    with pytest.raises(asyncio.CancelledError, match="blah"):
         async with create_task_group() as tg:
             task = await tg.start(taskfunc)
             tg.start_soon(sleep, 5)
             await wait_all_tasks_blocked()
             assert isinstance(task, asyncio.Task)
-            task.cancel('blah')
+            task.cancel("blah")
 
 
 async def test_start_soon_parent_id() -> None:
     root_task_id = get_current_task().id
-    parent_id: Optional[int] = None
+    parent_id: int | None = None
 
     async def subtask() -> None:
         nonlocal parent_id
@@ -1025,9 +1093,9 @@ async def test_start_soon_parent_id() -> None:
 
 async def test_start_parent_id() -> None:
     root_task_id = get_current_task().id
-    starter_task_id: Optional[int] = None
-    initial_parent_id: Optional[int] = None
-    permanent_parent_id: Optional[int] = None
+    starter_task_id: int | None = None
+    initial_parent_id: int | None = None
+    permanent_parent_id: int | None = None
 
     async def subtask(*, task_status: TaskStatus) -> None:
         nonlocal initial_parent_id, permanent_parent_id
@@ -1046,3 +1114,74 @@ async def test_start_parent_id() -> None:
     assert initial_parent_id != permanent_parent_id
     assert initial_parent_id == starter_task_id
     assert permanent_parent_id == root_task_id
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 11),
+    reason="Task uncancelling is only supported on Python 3.11",
+)
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+class TestUncancel:
+    async def test_uncancel_after_native_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with pytest.raises(asyncio.CancelledError), CancelScope():
+            task.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 1
+        task.uncancel()
+
+    async def test_uncancel_after_scope_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with CancelScope() as scope:
+            scope.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 0
+
+    async def test_uncancel_after_scope_and_native_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with pytest.raises(asyncio.CancelledError), CancelScope() as scope:
+            scope.cancel()
+            task.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 1
+        task.uncancel()
+
+
+class TestTaskStatusTyping:
+    """
+    These tests do not do anything at run time, but since the test suite is also checked
+    with a static type checker, it ensures that the `TaskStatus` typing works as
+    intended.
+    """
+
+    async def typetest_None(*, task_status: TaskStatus[None]) -> None:
+        task_status.started()
+        task_status.started(None)
+
+    async def typetest_None_Union(*, task_status: TaskStatus[int | None]) -> None:
+        task_status.started()
+        task_status.started(None)
+
+    async def typetest_non_None(*, task_status: TaskStatus[int]) -> None:
+        # We use `type: ignore` and `--warn-unused-ignores` to get type checking errors
+        # if these ever stop failing.
+        task_status.started()  # type: ignore[call-arg]
+        task_status.started(None)  # type: ignore[arg-type]
+
+    async def typetest_variance_good(*, task_status: TaskStatus[float]) -> None:
+        task_status2: TaskStatus[int] = task_status
+        task_status2.started(int())
+
+    async def typetest_variance_bad(*, task_status: TaskStatus[int]) -> None:
+        # We use `type: ignore` and `--warn-unused-ignores` to get type checking errors
+        # if these ever stop failing.
+        task_status2: TaskStatus[float] = task_status  # type: ignore[assignment]
+        task_status2.started(float())
+
+    async def typetest_optional_status(
+        *, task_status: TaskStatus[int] = TASK_STATUS_IGNORED
+    ) -> None:
+        task_status.started(1)
